@@ -268,21 +268,57 @@ void admin_send_response (xmlDocPtr doc, client_t *client,
     {
         xmlChar *buff = NULL;
         int len = 0;
-        unsigned int buf_len;
+        size_t buf_len;
+        ssize_t ret;
+
         xmlDocDumpMemory(doc, &buff, &len);
-        buf_len = len + 256 /* just a random medium number */;
+
+        buf_len = len + 1024;
+        if (buf_len < 4096)
+            buf_len = 4096;
 
         client_set_queue (client, NULL);
         client->refbuf = refbuf_new (buf_len);
 
-        /* FIXME: in this section we hope no function will ever return -1 */
-	len = util_http_build_header(client->refbuf->data, buf_len, 0,
+	ret = util_http_build_header(client->refbuf->data, buf_len, 0,
 	                             0, 200, NULL,
 				     "text/xml", "utf-8",
-				     NULL);
-	len += snprintf (client->refbuf->data + len, buf_len - len, "Content-Length: %d\r\n\r\n%s", xmlStrlen(buff), buff);
+				     NULL, NULL);
+        if (ret == -1) {
+            ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
+            client_send_500(client, "Header generation failed.");
+            xmlFree(buff);
+            return;
+        } else if (buf_len < (len + ret + 64)) {
+            void *new_data;
+            buf_len = ret + len + 64;
+            new_data = realloc(client->refbuf->data, buf_len);
+            if (new_data) {
+                ICECAST_LOG_DEBUG("Client buffer reallocation succeeded.");
+                client->refbuf->data = new_data;
+                client->refbuf->len = buf_len;
+                ret = util_http_build_header(client->refbuf->data, buf_len, 0,
+                                             0, 200, NULL,
+                                             "text/xml", "utf-8",
+                                             NULL, NULL);
+                if (ret == -1) {
+                    ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
+                    client_send_500(client, "Header generation failed.");
+                    xmlFree(buff);
+                    return;
+                }
+            } else {
+                ICECAST_LOG_ERROR("Client buffer reallocation failed. Dropping client.");
+                client_send_500(client, "Buffer reallocation failed.");
+                xmlFree(buff);
+                return;
+            } 
+        }
 
-        client->refbuf->len = len;
+        /* FIXME: in this section we hope no function will ever return -1 */
+	ret += snprintf (client->refbuf->data + ret, buf_len - ret, "Content-Length: %d\r\n\r\n%s", xmlStrlen(buff), buff);
+
+        client->refbuf->len = ret;
         xmlFree(buff);
         client->respcode = 200;
         fserve_add_client (client, NULL);
@@ -300,7 +336,7 @@ void admin_send_response (xmlDocPtr doc, client_t *client,
             config->adminroot_dir, PATH_SEPARATOR, xslt_template);
         config_release_config();
 
-        DEBUG1("Sending XSLT (%s)", fullpath_xslt_template);
+        ICECAST_LOG_DEBUG("Sending XSLT (%s)", fullpath_xslt_template);
         xslt_transform(doc, fullpath_xslt_template, client);
         free(fullpath_xslt_template);
     }
@@ -312,10 +348,10 @@ void admin_handle_request(client_t *client, const char *uri)
     const char *mount, *command_string;
     int command;
 
-    DEBUG1("Admin request (%s)", uri);
+    ICECAST_LOG_DEBUG("Admin request (%s)", uri);
     if (!((strcmp(uri, "/admin.cgi") == 0) ||
          (strncmp("/admin/", uri, 7) == 0))) {
-        ERROR0("Internal error: admin request isn't");
+        ICECAST_LOG_ERROR("Internal error: admin request isn't");
         client_send_401(client);
         return;
     }
@@ -327,11 +363,11 @@ void admin_handle_request(client_t *client, const char *uri)
         command_string = uri + 7;
     }
 
-    DEBUG1("Got command (%s)", command_string);
+    ICECAST_LOG_DEBUG("Got command (%s)", command_string);
     command = admin_get_command(command_string);
 
     if(command < 0) {
-        ERROR1("Error parsing command string or unrecognised command: %s",
+        ICECAST_LOG_ERROR("Error parsing command string or unrecognised command: %s",
                 command_string);
         client_send_400(client, "Unrecognised command");
         return;
@@ -383,7 +419,7 @@ void admin_handle_request(client_t *client, const char *uri)
                 case 0:
                     break;
                 default:
-                    INFO1("Bad or missing password on mount modification admin "
+                    ICECAST_LOG_INFO("Bad or missing password on mount modification admin "
                             "request (command: %s)", command_string);
                     client_send_401(client);
                     /* fall through */
@@ -397,7 +433,7 @@ void admin_handle_request(client_t *client, const char *uri)
 
         if (source == NULL)
         {
-            WARN2("Admin command %s on non-existent source %s", 
+            ICECAST_LOG_WARN("Admin command %s on non-existent source %s", 
                     command_string, mount);
             avl_tree_unlock(global.source_tree);
             client_send_400(client, "Source does not exist");
@@ -407,7 +443,7 @@ void admin_handle_request(client_t *client, const char *uri)
             if (source->running == 0 && source->on_demand == 0)
             {
                 avl_tree_unlock (global.source_tree);
-                INFO2("Received admin command %s on unavailable mount \"%s\"",
+                ICECAST_LOG_INFO("Received admin command %s on unavailable mount \"%s\"",
                         command_string, mount);
                 client_send_400 (client, "Source is not available");
                 return;
@@ -416,12 +452,12 @@ void admin_handle_request(client_t *client, const char *uri)
                     source->shoutcast_compat == 0)
             {
                 avl_tree_unlock (global.source_tree);
-                ERROR0 ("illegal change of metadata on non-shoutcast "
+                ICECAST_LOG_ERROR("illegal change of metadata on non-shoutcast "
                         "compatible stream");
                 client_send_400 (client, "illegal metadata call");
                 return;
             }
-            INFO2("Received admin command %s on mount \"%s\"", 
+            ICECAST_LOG_INFO("Received admin command %s on mount \"%s\"", 
                     command_string, mount);
             admin_handle_mount_request(client, source, command);
             avl_tree_unlock(global.source_tree);
@@ -434,7 +470,7 @@ void admin_handle_request(client_t *client, const char *uri)
            mounts from the master, so handle this request
            validating against the relay password */
             if(!connection_check_relay_pass(client->parser)) {
-                INFO1("Bad or missing password on admin command "
+                ICECAST_LOG_INFO("Bad or missing password on admin command "
                       "request (command: %s)", command_string);
                 client_send_401(client);
                 return;
@@ -442,7 +478,7 @@ void admin_handle_request(client_t *client, const char *uri)
         }
         else {
             if(!connection_check_admin_pass(client->parser)) {
-                INFO1("Bad or missing password on admin command "
+                ICECAST_LOG_INFO("Bad or missing password on admin command "
                       "request (command: %s)", command_string);
                 client_send_401(client);
                 return;
@@ -481,7 +517,7 @@ static void admin_handle_general_request(client_t *client, int command)
             command_list_mounts(client, TRANSFORMED);
             break;
         default:
-            WARN0("General admin request not recognised");
+            ICECAST_LOG_WARN("General admin request not recognised");
             client_send_400(client, "Unknown admin request");
             return;
     }
@@ -549,7 +585,7 @@ static void admin_handle_mount_request(client_t *client, source_t *source,
             command_updatemetadata(client, source, RAW);
             break;
         default:
-            WARN0("Mount request not recognised");
+            ICECAST_LOG_WARN("Mount request not recognised");
             client_send_400(client, "Mount request unknown");
             break;
     }
@@ -573,7 +609,14 @@ static void html_success(client_t *client, char *message)
     ret = util_http_build_header(client->refbuf->data, PER_CLIENT_REFBUF_SIZE, 0,
                                  0, 200, NULL,
 				 "text/html", "utf-8",
-				 "");
+				 "", NULL);
+
+    if (ret == -1 || ret >= PER_CLIENT_REFBUF_SIZE) {
+        ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
+        client_send_500(client, "Header generation failed.");
+        return;
+    }
+
     snprintf(client->refbuf->data + ret, PER_CLIENT_REFBUF_SIZE - ret,
              "<html><head><title>Admin request successful</title></head>"
 	     "<body><p>%s</p></body></html>", message);
@@ -594,11 +637,11 @@ static void command_move_clients(client_t *client, source_t *source,
     char buf[255];
     int parameters_passed = 0;
 
-    DEBUG0("Doing optional check");
+    ICECAST_LOG_DEBUG("Doing optional check");
     if((COMMAND_OPTIONAL(client, "destination", dest_source))) {
         parameters_passed = 1;
     }
-    DEBUG1("Done optional check (%d)", parameters_passed);
+    ICECAST_LOG_DEBUG("Done optional check (%d)", parameters_passed);
     if (!parameters_passed) {
         doc = admin_build_sourcelist(source->mount);
         admin_send_response(doc, client, response, 
@@ -627,7 +670,7 @@ static void command_move_clients(client_t *client, source_t *source,
         return;
     }
 
-    INFO2 ("source is \"%s\", destination is \"%s\"", source->mount, dest->mount);
+    ICECAST_LOG_INFO("source is \"%s\", destination is \"%s\"", source->mount, dest->mount);
 
     doc = xmlNewDoc (XMLSTR("1.0"));
     node = xmlNewDocNode(doc, NULL, XMLSTR("iceresponse"), NULL);
@@ -712,7 +755,14 @@ static void command_buildm3u(client_t *client,  const char *mount)
     ret = util_http_build_header(client->refbuf->data, PER_CLIENT_REFBUF_SIZE, 0,
                                  0, 200, NULL,
 				 "audio/x-mpegurl", NULL,
-				 NULL);
+				 NULL, NULL);
+
+    if (ret == -1 || ret >= (PER_CLIENT_REFBUF_SIZE - 512)) { /* we want at least 512 Byte left for data */
+        ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
+        client_send_500(client, "Header generation failed.");
+        return;
+    }
+
 
     config = config_get_config();
     snprintf (client->refbuf->data + ret, PER_CLIENT_REFBUF_SIZE - ret,
@@ -748,7 +798,7 @@ static void command_manageauth(client_t *client, source_t *source,
     {
         if (mountinfo == NULL || mountinfo->auth == NULL)
         {
-            WARN1 ("manage auth request for %s but no facility available", source->mount);
+            ICECAST_LOG_WARN("manage auth request for %s but no facility available", source->mount);
             break;
         }
         COMMAND_OPTIONAL(client, "action", action);
@@ -764,7 +814,7 @@ static void command_manageauth(client_t *client, source_t *source,
 
             if (username == NULL || password == NULL)
             {
-                WARN1 ("manage auth request add for %s but no user/pass", source->mount);
+                ICECAST_LOG_WARN("manage auth request add for %s but no user/pass", source->mount);
                 break;
             }
             ret = mountinfo->auth->adduser(mountinfo->auth, username, password);
@@ -782,7 +832,7 @@ static void command_manageauth(client_t *client, source_t *source,
         {
             if (username == NULL)
             {
-                WARN1 ("manage auth request delete for %s but no username", source->mount);
+                ICECAST_LOG_WARN("manage auth request delete for %s but no username", source->mount);
                 break;
             }
             ret = mountinfo->auth->deleteuser(mountinfo->auth, username);
@@ -860,10 +910,10 @@ static void command_kill_client(client_t *client, source_t *source,
     doc = xmlNewDoc (XMLSTR("1.0"));
     node = xmlNewDocNode(doc, NULL, XMLSTR("iceresponse"), NULL);
     xmlDocSetRootElement(doc, node);
-    DEBUG1("Response is %d", response);
+    ICECAST_LOG_DEBUG("Response is %d", response);
 
     if(listener != NULL) {
-        INFO1("Admin request: client %d removed", id);
+        ICECAST_LOG_INFO("Admin request: client %d removed", id);
 
         /* This tags it for removal on the next iteration of the main source
          * loop
@@ -891,7 +941,7 @@ static void command_fallback(client_t *client, source_t *source,
     const char *fallback;
     char *old;
 
-    DEBUG0("Got fallback request");
+    ICECAST_LOG_DEBUG("Got fallback request");
 
     COMMAND_REQUIRE(client, "fallback", fallback);
 
@@ -916,7 +966,7 @@ static void command_metadata(client_t *client, source_t *source,
     node = xmlNewDocNode (doc, NULL, XMLSTR("iceresponse"), NULL);
     xmlDocSetRootElement(doc, node);
 
-    DEBUG0("Got metadata update request");
+    ICECAST_LOG_DEBUG("Got metadata update request");
 
     COMMAND_REQUIRE(client, "mode", action);
     COMMAND_OPTIONAL(client, "song", song);
@@ -944,7 +994,7 @@ static void command_metadata(client_t *client, source_t *source,
         if (song)
         {
             plugin->set_tag (plugin, "song", song, charset);
-            INFO2 ("Metadata on mountpoint %s changed to \"%s\"", source->mount, song);
+            ICECAST_LOG_INFO("Metadata on mountpoint %s changed to \"%s\"", source->mount, song);
         }
         else
         {
@@ -952,7 +1002,7 @@ static void command_metadata(client_t *client, source_t *source,
             {
                 plugin->set_tag (plugin, "title", title, charset);
                 plugin->set_tag (plugin, "artist", artist, charset);
-                INFO3("Metadata on mountpoint %s changed to \"%s - %s\"",
+                ICECAST_LOG_INFO("Metadata on mountpoint %s changed to \"%s - %s\"",
                         source->mount, artist, title);
             }
         }
@@ -983,7 +1033,7 @@ static void command_shoutcast_metadata(client_t *client, source_t *source)
     const char *value;
     int same_ip = 1;
 
-    DEBUG0("Got shoutcast metadata update request");
+    ICECAST_LOG_DEBUG("Got shoutcast metadata update request");
 
     COMMAND_REQUIRE(client, "mode", action);
     COMMAND_REQUIRE(client, "song", value);
@@ -1002,7 +1052,7 @@ static void command_shoutcast_metadata(client_t *client, source_t *source)
         source->format->set_tag (source->format, "title", value, NULL);
         source->format->set_tag (source->format, NULL, NULL, NULL);
 
-        DEBUG2("Metadata on mountpoint %s changed to \"%s\"", 
+        ICECAST_LOG_DEBUG("Metadata on mountpoint %s changed to \"%s\"", 
                 source->mount, value);
         html_success(client, "Metadata update successful");
     }
@@ -1015,7 +1065,7 @@ static void command_shoutcast_metadata(client_t *client, source_t *source)
 static void command_stats(client_t *client, const char *mount, int response) {
     xmlDocPtr doc;
 
-    DEBUG0("Stats request, sending xml stats");
+    ICECAST_LOG_DEBUG("Stats request, sending xml stats");
 
     doc = stats_get_xml(1, mount);
     admin_send_response(doc, client, response, STATS_TRANSFORMED_REQUEST);
@@ -1025,14 +1075,21 @@ static void command_stats(client_t *client, const char *mount, int response) {
 
 static void command_list_mounts(client_t *client, int response)
 {
-    DEBUG0("List mounts request");
+    ICECAST_LOG_DEBUG("List mounts request");
 
     if (response == PLAINTEXT)
     {
-        util_http_build_header(client->refbuf->data, PER_CLIENT_REFBUF_SIZE, 0,
+        ssize_t ret = util_http_build_header(client->refbuf->data, PER_CLIENT_REFBUF_SIZE, 0,
 	                       0, 200, NULL,
 			       "text/plain", "utf-8",
-			       "");
+			       "", NULL);
+
+        if (ret == -1 || ret >= PER_CLIENT_REFBUF_SIZE) {
+            ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
+            client_send_500(client, "Header generation failed.");
+            return;
+        }
+
         client->refbuf->len = strlen (client->refbuf->data);
         client->respcode = 200;
 
