@@ -109,7 +109,7 @@ static void auth_url_clear(auth_t *self)
 {
     auth_url *url;
 
-    INFO0 ("Doing auth URL cleanup");
+    ICECAST_LOG_INFO("Doing auth URL cleanup");
     url = self->state;
     self->state = NULL;
     curl_easy_cleanup (url->handle);
@@ -263,7 +263,7 @@ static auth_result url_remove_listener (auth_client *auth_user)
     curl_easy_setopt (url->handle, CURLOPT_WRITEHEADER, auth_user);
 
     if (curl_easy_perform (url->handle))
-        WARN2 ("auth to server %s failed with %s", url->removeurl, url->errormsg);
+        ICECAST_LOG_WARN("auth to server %s failed with %s", url->removeurl, url->errormsg);
 
     free (userpwd);
 
@@ -394,13 +394,13 @@ static auth_result url_add_listener (auth_client *auth_user)
 
     if (res)
     {
-        WARN2 ("auth to server %s failed with %s", url->addurl, url->errormsg);
+        ICECAST_LOG_WARN("auth to server %s failed with %s", url->addurl, url->errormsg);
         return AUTH_FAILED;
     }
     /* we received a response, lets see what it is */
     if (client->authenticated)
         return AUTH_OK;
-    INFO2 ("client auth (%s) failed with \"%s\"", url->addurl, url->errormsg);
+    ICECAST_LOG_INFO("client auth (%s) failed with \"%s\"", url->addurl, url->errormsg);
     return AUTH_FAILED;
 }
 
@@ -452,7 +452,7 @@ static void url_stream_start (auth_client *auth_user)
     curl_easy_setopt (url->handle, CURLOPT_WRITEHEADER, auth_user);
 
     if (curl_easy_perform (url->handle))
-        WARN2 ("auth to server %s failed with %s", stream_start_url, url->errormsg);
+        ICECAST_LOG_WARN("auth to server %s failed with %s", stream_start_url, url->errormsg);
 
     auth_release (auth);
     free (stream_start_url);
@@ -504,7 +504,7 @@ static void url_stream_end (auth_client *auth_user)
     curl_easy_setopt (url->handle, CURLOPT_WRITEHEADER, auth_user);
 
     if (curl_easy_perform (url->handle))
-        WARN2 ("auth to server %s failed with %s", stream_end_url, url->errormsg);
+        ICECAST_LOG_WARN("auth to server %s failed with %s", stream_end_url, url->errormsg);
 
     auth_release (auth);
     free (stream_end_url);
@@ -539,10 +539,20 @@ static void url_stream_auth (auth_client *auth_user)
     host = util_url_escape (config->hostname);
     port = config->port;
     config_release_config ();
-    user = util_url_escape (client->username);
-    pass = util_url_escape (client->password);
     ipaddr = util_url_escape (client->con->ip);
 
+    if (client->username) {
+        user = util_url_escape(client->username);
+    } else {
+        user = strdup("");
+    }
+
+    if (client->password) {
+        pass = util_url_escape(client->password);
+    } else {
+        pass = strdup("");
+    }
+
     snprintf (post, sizeof (post),
             "action=stream_auth&mount=%s&ip=%s&server=%s&port=%d&user=%s&pass=%s%s",
             mount, ipaddr, host, port, user, pass, admin);
@@ -554,7 +564,7 @@ static void url_stream_auth (auth_client *auth_user)
 
     client->authenticated = 0;
     if (curl_easy_perform (url->handle))
-        WARN2 ("auth to server %s failed with %s", url->stream_auth, url->errormsg);
+        ICECAST_LOG_WARN("auth to server %s failed with %s", url->stream_auth, url->errormsg);
 }
 
 
@@ -683,7 +693,7 @@ int auth_get_url_auth (auth_t *authenticator, config_options_t *options)
         snprintf (url_info->userpwd, len, "%s:%s", url_info->username, url_info->password);
     }
 
-    INFO0("URL based authentication setup");
+    ICECAST_LOG_INFO("URL based authentication setup");
     return 0;
 }
 
