@@ -8,6 +8,7 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2011-2012, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 /* client.c
@@ -37,6 +38,8 @@
 #include "client.h"
 #include "logging.h"
 
+#include "util.h"
+
 #ifdef _WIN32
 #define snprintf _snprintf
 #endif
@@ -62,7 +65,7 @@ int client_create (client_t **c_ptr, connection_t *con, http_parser_t *parser)
 
     global.clients++;
     if (config->client_limit < global.clients)
-        WARN2 ("server client limit reached (%d/%d)", config->client_limit, global.clients);
+        ICECAST_LOG_WARN("server client limit reached (%d/%d)", config->client_limit, global.clients);
     else
         ret = 0;
 
@@ -130,7 +133,7 @@ int client_check_source_auth (client_t *client, const char *mount)
     char *pass = config->source_password;
     char *user = "source";
     int ret = -1;
-    mount_proxy *mountinfo = config_find_mount (config, mount);
+    mount_proxy *mountinfo = config_find_mount (config, mount, MOUNT_TYPE_NORMAL);
 
     do
     {
@@ -175,56 +178,77 @@ int client_read_bytes (client_t *client, void *buf, unsigned len)
     bytes = client->con->read (client->con, buf, len);
 
     if (bytes == -1 && client->con->error)
-        DEBUG0 ("reading from connection has failed");
+        ICECAST_LOG_DEBUG("reading from connection has failed");
 
     return bytes;
 }
 
-
-void client_send_400(client_t *client, char *message) {
-    snprintf (client->refbuf->data, PER_CLIENT_REFBUF_SIZE,
-            "HTTP/1.0 400 Bad Request\r\n"
-            "Content-Type: text/html\r\n\r\n"
-            "<b>%s</b>\r\n", message);
-    client->respcode = 400;
-    client->refbuf->len = strlen (client->refbuf->data);
-    fserve_add_client (client, NULL);
-}
-
-void client_send_404(client_t *client, char *message) {
-
-    snprintf (client->refbuf->data, PER_CLIENT_REFBUF_SIZE,
-            "HTTP/1.0 404 File Not Found\r\n"
-            "Content-Type: text/html\r\n\r\n"
-            "<b>%s</b>\r\n", message);
-    client->respcode = 404;
-    client->refbuf->len = strlen (client->refbuf->data);
-    fserve_add_client (client, NULL);
-}
-
-
-void client_send_401(client_t *client) {
-    snprintf (client->refbuf->data, PER_CLIENT_REFBUF_SIZE,
-            "HTTP/1.0 401 Authentication Required\r\n"
-            "WWW-Authenticate: Basic realm=\"Icecast2 Server\"\r\n"
-            "\r\n"
-            "You need to authenticate\r\n");
-    client->respcode = 401;
-    client->refbuf->len = strlen (client->refbuf->data);
-    fserve_add_client (client, NULL);
-}
-
-void client_send_403(client_t *client, const char *reason)
+static void client_send_error(client_t *client, int status, int plain, const char *message)
 {
-    if (reason == NULL)
-        reason = "Forbidden";
-    snprintf (client->refbuf->data, PER_CLIENT_REFBUF_SIZE,
-            "HTTP/1.0 403 %s\r\n\r\n", reason);
-    client->respcode = 403;
+    ssize_t ret;
+
+    ret = util_http_build_header(client->refbuf->data, PER_CLIENT_REFBUF_SIZE, 0,
+                                 0, status, NULL,
+                                 plain ? "text/plain" : "text/html", "utf-8",
+                                 plain ? message : "", NULL);
+
+    if (ret == -1 || ret >= PER_CLIENT_REFBUF_SIZE) {
+        ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
+        client_send_500(client, "Header generation failed.");
+        return;
+    }
+
+    if (!plain)
+        snprintf(client->refbuf->data + ret, PER_CLIENT_REFBUF_SIZE - ret,
+                 "<html><head><title>Error %i</title></head><body><b>%i - %s</b></body></html>\r\n",
+                 status, status, message);
+
+    client->respcode = status;
     client->refbuf->len = strlen (client->refbuf->data);
     fserve_add_client (client, NULL);
 }
 
+void client_send_100(client_t *client)
+{
+    /* On demand inject a HTTP/1.1 100 Continue to make sure clients are happy */
+    sock_write (client->con->sock, "HTTP/1.1 100 Continue\r\n\r\n");
+}
+
+void client_send_400(client_t *client, const char *message)
+{
+    client_send_error(client, 400, 0, message);
+}
+
+void client_send_404(client_t *client, const char *message)
+{
+    client_send_error(client, 404, 0, message);
+}
+
+void client_send_401(client_t *client)
+{
+    client_send_error(client, 401, 1, "You need to authenticate\r\n");
+}
+
+void client_send_403(client_t *client, const char *message)
+{
+    client_send_error(client, 403, 1, message);
+}
+
+/* this function is designed to work even if client is in bad state */
+void client_send_500(client_t *client, const char *message) {
+    const char header[] = "HTTP/1.0 500 Internal Server Error\r\nContent-Type: text/plain; charset=utf-8\r\n\r\n"
+                          "500 - Internal Server Error\n---------------------------\n";
+    const size_t header_len = sizeof(header) - 1;
+    int ret;
+
+    ret = client_send_bytes(client, header, header_len);
+
+    /* only send message if we have one AND if header could have transmitted completly */
+    if (message && ret == header_len)
+        client_send_bytes(client, message, strlen(message));
+
+    client_destroy(client);
+}
 
 /* helper function for sending the data to a client */
 int client_send_bytes (client_t *client, const void *buf, unsigned len)
@@ -232,7 +256,7 @@ int client_send_bytes (client_t *client, const void *buf, unsigned len)
     int ret = client->con->send (client->con, buf, len);
 
     if (client->con->error)
-        DEBUG0 ("Client connection died");
+        ICECAST_LOG_DEBUG("Client connection died");
 
     return ret;
 }
