@@ -40,8 +40,10 @@
 #define PRI_OFF_T "ld"
 #define snprintf _snprintf
 #define strncasecmp _strnicmp
+#ifndef S_ISREG
 #define S_ISREG(mode)  ((mode) & _S_IFREG)
 #endif
+#endif
 
 #include "thread/thread.h"
 #include "avl/avl.h"
@@ -106,7 +108,7 @@ void fserve_initialize(void)
     config_release_config();
 
     stats_event (NULL, "file_connections", "0");
-    INFO0("file serving started");
+    ICECAST_LOG_INFO("file serving started");
 }
 
 void fserve_shutdown(void)
@@ -132,7 +134,7 @@ void fserve_shutdown(void)
 
     thread_spin_unlock (&pending_lock);
     thread_spin_destroy (&pending_lock);
-    INFO0("file serving stopped");
+    ICECAST_LOG_INFO("file serving stopped");
 }
 
 #ifdef HAVE_POLL
@@ -330,7 +332,7 @@ static void *fserv_thread_function(void *arg)
             fclient = fclient->next;
         }
     }
-    DEBUG0 ("fserve handler exit");
+    ICECAST_LOG_DEBUG("fserve handler exit");
     return NULL;
 }
 
@@ -412,7 +414,7 @@ int fserve_client_create (client_t *httpclient, const char *path)
     FILE *file;
 
     fullpath = util_get_path_from_normalised_uri (path);
-    INFO2 ("checking for file %H (%H)", path, fullpath);
+    ICECAST_LOG_INFO("checking for file %H (%H)", path, fullpath);
 
     if (strcmp (util_get_extension (fullpath), "m3u") == 0)
         m3u_requested = 1;
@@ -429,7 +431,7 @@ int fserve_client_create (client_t *httpclient, const char *path)
         /* the m3u can be generated, but send an m3u file if available */
         if (m3u_requested == 0 && xslt_playlist_requested == NULL)
         {
-            WARN2 ("req for file \"%H\" %s", fullpath, strerror (errno));
+            ICECAST_LOG_WARN("req for file \"%H\" %s", fullpath, strerror (errno));
             client_send_404 (httpclient, "The file you requested could not be found");
             free (fullpath);
             return -1;
@@ -456,7 +458,12 @@ int fserve_client_create (client_t *httpclient, const char *path)
         httpclient->respcode = 200;
         ret = util_http_build_header (httpclient->refbuf->data, BUFSIZE, 0,
 	                              0, 200, NULL,
-				      "audio/x-mpegurl", NULL, "");
+				      "audio/x-mpegurl", NULL, "", NULL);
+        if (ret == -1 || ret >= (BUFSIZE - 512)) { /* we want at least 512 bytes left for the content of the playlist */
+            ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
+            client_send_500(httpclient, "Header generation failed.");
+            return -1;
+        }
         if (host == NULL)
         {
 	    config = config_get_config();
@@ -499,7 +506,7 @@ int fserve_client_create (client_t *httpclient, const char *path)
     config = config_get_config();
     if (config->fileserve == 0)
     {
-        DEBUG1 ("on demand file \"%H\" refused", fullpath);
+        ICECAST_LOG_DEBUG("on demand file \"%H\" refused. Serving static files has been disabled in the config", fullpath);
         client_send_404 (httpclient, "The file you requested could not be found");
         config_release_config();
         free (fullpath);
@@ -510,7 +517,7 @@ int fserve_client_create (client_t *httpclient, const char *path)
     if (S_ISREG (file_buf.st_mode) == 0)
     {
         client_send_404 (httpclient, "The file you requested could not be found");
-        WARN1 ("found requested file but there is no handler for it: %H", fullpath);
+        ICECAST_LOG_WARN("found requested file but there is no handler for it: %H", fullpath);
         free (fullpath);
         return -1;
     }
@@ -518,7 +525,7 @@ int fserve_client_create (client_t *httpclient, const char *path)
     file = fopen (fullpath, "rb");
     if (file == NULL)
     {
-        WARN1 ("Problem accessing file \"%H\"", fullpath);
+        ICECAST_LOG_WARN("Problem accessing file \"%H\"", fullpath);
         client_send_404 (httpclient, "File not readable");
         free (fullpath);
         return -1;
@@ -565,7 +572,12 @@ int fserve_client_create (client_t *httpclient, const char *path)
 		bytes = util_http_build_header (httpclient->refbuf->data, BUFSIZE, 0,
 		                                0, 206, NULL,
 						type, NULL,
-						NULL);
+						NULL, NULL);
+                if (bytes == -1 || bytes >= (BUFSIZE - 512)) { /* we want at least 512 bytes left */
+                    ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
+                    client_send_500(httpclient, "Header generation failed.");
+                    return -1;
+                }
                 bytes += snprintf (httpclient->refbuf->data + bytes, BUFSIZE - bytes,
                     "Accept-Ranges: bytes\r\n"
                     "Content-Length: %" PRI_OFF_T "\r\n"
@@ -591,7 +603,12 @@ int fserve_client_create (client_t *httpclient, const char *path)
 	bytes = util_http_build_header (httpclient->refbuf->data, BUFSIZE, 0,
 	                                0, 200, NULL,
 					type, NULL,
-					NULL);
+					NULL, NULL);
+        if (bytes == -1 || bytes >= (BUFSIZE - 512)) { /* we want at least 512 bytes left */
+            ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
+            client_send_500(httpclient, "Header generation failed.");
+            return -1;
+        }
         bytes += snprintf (httpclient->refbuf->data + bytes, BUFSIZE - bytes,
             "Accept-Ranges: bytes\r\n"
             "Content-Length: %" PRI_OFF_T "\r\n\r\n",
@@ -627,7 +644,7 @@ static void fserve_add_pending (fserve_t *fclient)
     if (run_fserv == 0)
     {
         run_fserv = 1;
-        DEBUG0 ("fserve handler waking up");
+        ICECAST_LOG_DEBUG("fserve handler waking up");
         thread_create("File Serving Thread", fserv_thread_function, NULL, THREAD_DETACHED);
     }
     thread_spin_unlock (&pending_lock);
@@ -641,7 +658,7 @@ int fserve_add_client (client_t *client, FILE *file)
 {
     fserve_t *fclient = calloc (1, sizeof(fserve_t));
 
-    DEBUG0 ("Adding client to file serving engine");
+    ICECAST_LOG_DEBUG("Adding client to file serving engine");
     if (fclient == NULL)
     {
         client_send_404 (client, "memory exhausted");
@@ -663,7 +680,7 @@ void fserve_add_client_callback (client_t *client, fserve_callback_t callback, v
 {
     fserve_t *fclient = calloc (1, sizeof(fserve_t));
 
-    DEBUG0 ("Adding client to file serving engine");
+    ICECAST_LOG_DEBUG("Adding client to file serving engine");
     if (fclient == NULL)
     {
         client_send_404 (client, "memory exhausted");
@@ -708,7 +725,7 @@ void fserve_recheck_mime_types (ice_config_t *config)
     mimefile = fopen (config->mimetypes_fn, "r");
     if (mimefile == NULL)
     {
-        WARN1 ("Cannot open mime types file %s", config->mimetypes_fn);
+        ICECAST_LOG_WARN("Cannot open mime types file %s", config->mimetypes_fn);
         return;
     }
 
