@@ -8,7 +8,7 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
- * Copyright 2011-2012, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
+ * Copyright 2012-2014, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,6 +45,7 @@
 #include "refbuf.h"
 #include "connection.h"
 #include "client.h"
+#include "source.h"
 
 #define CATMODULE "util"
 
@@ -266,21 +267,25 @@ static char safechars[256] = {
 
 char *util_url_escape (const char *src)
 {
-    int len = strlen(src);
-    /* Efficiency not a big concern here, keep the code simple/conservative */
-    char *dst = calloc(1, len*3 + 1); 
+    size_t len;
+    char *dst; 
     unsigned char *source = (unsigned char *)src;
-    int i,j=0;
+    size_t i, j;
 
-    for(i=0; i < len; i++) {
+    if (!src)
+        return NULL;
+
+    len = strlen(src);
+    /* Efficiency not a big concern here, keep the code simple/conservative */
+    dst = calloc(1, len*3 + 1);
+
+    for(i = 0, j = 0; i < len; i++) {
         if(safechars[source[i]]) {
             dst[j++] = source[i];
-        }
-        else {
-            dst[j] = '%';
-            dst[j+1] = hexchars[ (source[i] >> 4) & 0xf ];
-            dst[j+2] = hexchars[ source[i] & 0xf ];
-            j+= 3;
+        } else {
+            dst[j++] = '%';
+            dst[j++] = hexchars[(source[i] >> 4) & 0x0F];
+            dst[j++] = hexchars[ source[i]       & 0x0F];
         }
     }
 
@@ -319,7 +324,7 @@ char *util_url_unescape (const char *src)
                 done = 1;
                 break;
             case 0:
-                ERROR0("Fatal internal logic error in util_url_unescape()");
+                ICECAST_LOG_ERROR("Fatal internal logic error in util_url_unescape()");
                 free(decoded);
                 return NULL;
                 break;
@@ -350,7 +355,7 @@ char *util_normalise_uri(const char *uri) {
     path = util_url_unescape(uri);
 
     if(path == NULL) {
-        WARN1("Error decoding URI: %s\n", uri);
+        ICECAST_LOG_WARN("Error decoding URI: %s\n", uri);
         return NULL;
     }
 
@@ -358,7 +363,7 @@ char *util_normalise_uri(const char *uri) {
     if(verify_path(path))
         return path;
     else {
-        WARN1("Rejecting invalid path \"%s\"", path);
+        ICECAST_LOG_WARN("Rejecting invalid path \"%s\"", path);
         free(path);
         return NULL;
     }
@@ -486,11 +491,70 @@ char *util_base64_decode(const char *data)
     return result;
 }
 
+/* TODO, FIXME: handle memory allocation errors better. */
+static inline void   _build_headers_loop(char **ret, size_t *len, ice_config_http_header_t *header, int status) {
+    size_t headerlen;
+    const char *name;
+    const char *value;
+    char * r = *ret;
+
+    if (!header)
+        return;
+
+    do {
+        /* filter out header's we don't use. */
+        if (header->status != 0 && header->status != status) continue;
+
+        /* get the name of the header */
+        name = header->name;
+
+        /* handle type of the header */
+        value = NULL;
+        switch (header->type) {
+            case HTTP_HEADER_TYPE_STATIC:
+                value = header->value;
+                break;
+        }
+
+        /* check data */
+        if (!name || !value)
+            continue;
+
+        /* append the header to the buffer */
+        headerlen = strlen(name) + strlen(value) + 4;
+        *len += headerlen;
+        r = realloc(r, *len);
+        strcat(r, name);
+        strcat(r, ": ");
+        strcat(r, value);
+        strcat(r, "\r\n");
+    } while ((header = header->next));
+    *ret = r;
+}
+static inline char * _build_headers(int status, ice_config_t *config, source_t *source) {
+    mount_proxy *mountproxy = NULL;
+    char *ret = NULL;
+    size_t len = 1;
+
+    if (source)
+        mountproxy = config_find_mount(config, source->mount, MOUNT_TYPE_NORMAL);
+
+    ret = calloc(1, 1);
+    *ret = 0;
+
+    _build_headers_loop(&ret, &len, config->http_headers, status);
+    if (mountproxy && mountproxy->http_headers)
+        _build_headers_loop(&ret, &len, mountproxy->http_headers, status);
+
+    return ret;
+}
+
 ssize_t util_http_build_header(char * out, size_t len, ssize_t offset,
         int cache,
         int status, const char * statusmsg,
         const char * contenttype, const char * charset,
-        const char * datablock) {
+        const char * datablock,
+        struct source_tag * source) {
     const char * http_version = "1.0";
     ice_config_t *config;
     time_t now;
@@ -500,6 +564,7 @@ ssize_t util_http_build_header(char * out, size_t len, ssize_t offset,
     char status_buffer[80];
     char contenttype_buffer[80];
     ssize_t ret;
+    char * extra_headers;
 
     if (!out)
         return -1;
@@ -563,7 +628,8 @@ ssize_t util_http_build_header(char * out, size_t len, ssize_t offset,
         currenttime_buffer[0] = '\0';
 
     config = config_get_config();
-    ret = snprintf (out, len, "%sServer: %s\r\n%s%s%s%s%s%s",
+    extra_headers = _build_headers(status, config, source);
+    ret = snprintf (out, len, "%sServer: %s\r\n%s%s%s%s%s%s%s",
                               status_buffer,
 			      config->server_id,
 			      currenttime_buffer,
@@ -572,8 +638,10 @@ ssize_t util_http_build_header(char * out, size_t len, ssize_t offset,
                               (cache     ? "" : "Cache-Control: no-cache\r\n"
                                                 "Expires: Mon, 26 Jul 1997 05:00:00 GMT\r\n"
                                                 "Pragma: no-cache\r\n"),
+                              extra_headers,
                               (datablock ? "\r\n" : ""),
                               (datablock ? datablock : ""));
+    free(extra_headers);
     config_release_config();
 
     return ret;
@@ -617,7 +685,7 @@ int util_dict_set(util_dict *dict, const char *key, const char *val)
     util_dict *prev;
 
     if (!dict || !key) {
-        ERROR0("NULL values passed to util_dict_set()");
+        ICECAST_LOG_ERROR("NULL values passed to util_dict_set()");
         return 0;
     }
 
@@ -632,7 +700,7 @@ int util_dict_set(util_dict *dict, const char *key, const char *val)
     if (!dict) {
         dict = util_dict_new();
         if (!dict) {
-            ERROR0("unable to allocate new dictionary");
+            ICECAST_LOG_ERROR("unable to allocate new dictionary");
             return 0;
         }
         if (prev)
@@ -646,13 +714,13 @@ int util_dict_set(util_dict *dict, const char *key, const char *val)
             prev->next = NULL;
         util_dict_free (dict);
 
-        ERROR0("unable to allocate new dictionary key");
+        ICECAST_LOG_ERROR("unable to allocate new dictionary key");
         return 0;
     }
 
     dict->val = strdup(val);
     if (!dict->val) {
-        ERROR0("unable to allocate new dictionary value");
+        ICECAST_LOG_ERROR("unable to allocate new dictionary value");
         return 0;
     }
 
@@ -750,7 +818,7 @@ char *util_conv_string (const char *string, const char *in_charset, const char *
         xmlBufferPtr utf8 = xmlBufferCreate ();
         xmlBufferPtr conv = xmlBufferCreate ();
 
-        INFO2 ("converting metadata from %s to %s", in_charset, out_charset);
+        ICECAST_LOG_INFO("converting metadata from %s to %s", in_charset, out_charset);
         xmlBufferCCat (orig, string);
         if (xmlCharEncInFunc (in, utf8, orig) > 0)
         {
