@@ -142,7 +142,7 @@ static xsltStylesheetPtr xslt_get_stylesheet(const char *fn) {
     struct stat file;
 
     if(stat(fn, &file)) {
-        WARN2("Error checking for stylesheet file \"%s\": %s", fn, 
+        ICECAST_LOG_WARN("Error checking for stylesheet file \"%s\": %s", fn, 
                 strerror(errno));
         return NULL;
     }
@@ -164,7 +164,7 @@ static xsltStylesheetPtr xslt_get_stylesheet(const char *fn) {
                     cache[i].stylesheet = xsltParseStylesheetFile (XMLSTR(fn));
                     cache[i].cache_age = time(NULL);
                 }
-                DEBUG1("Using cached sheet %i", i);
+                ICECAST_LOG_DEBUG("Using cached sheet %i", i);
                 return cache[i].stylesheet;
             }
         }
@@ -202,7 +202,7 @@ void xslt_transform(xmlDocPtr doc, const char *xslfilename, client_t *client)
     if (cur == NULL)
     {
         thread_mutex_unlock(&xsltlock);
-        ERROR1 ("problem reading stylesheet \"%s\"", xslfilename);
+        ICECAST_LOG_ERROR("problem reading stylesheet \"%s\"", xslfilename);
         client_send_404 (client, "Could not parse XSLT file");
         return;
     }
@@ -231,28 +231,57 @@ void xslt_transform(xmlDocPtr doc, const char *xslfilename, client_t *client)
     }
     if (problem == 0)
     {
-        /* the 100 is to allow for the hardcoded headers */
-        unsigned int full_len = strlen (mediatype) + len + 256;
-        refbuf_t *refbuf = refbuf_new (full_len);
-	ssize_t ret;
+        ssize_t ret;
+        int failed = 0;
+        refbuf_t *refbuf;
+        size_t full_len = strlen (mediatype) + len + 1024;
+        if (full_len < 4096)
+            full_len = 4096;
+        refbuf = refbuf_new (full_len);
 
         if (string == NULL)
             string = xmlCharStrdup ("");
-        ret = util_http_build_header(refbuf->data, full_len, 0, 0, 200, NULL, mediatype, charset, NULL);
-	snprintf (refbuf->data + ret, full_len - ret,
-                "Content-Length: %d\r\n\r\n%s",
-                len, string);
+        ret = util_http_build_header(refbuf->data, full_len, 0, 0, 200, NULL, mediatype, charset, NULL, NULL);
+        if (ret == -1) {
+            ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
+            client_send_500(client, "Header generation failed.");
+        } else {
+            if ( full_len < (ret + len + 64) ) {
+                void *new_data;
+                full_len = ret + len + 64;
+                new_data = realloc(refbuf->data, full_len);
+                if (new_data) {
+                    ICECAST_LOG_DEBUG("Client buffer reallocation succeeded.");
+                    refbuf->data = new_data;
+                    refbuf->len = full_len;
+                    ret = util_http_build_header(refbuf->data, full_len, 0, 0, 200, NULL, mediatype, charset, NULL, NULL);
+                    if (ret == -1) {
+                        ICECAST_LOG_ERROR("Dropping client as we can not build response headers.");
+                        client_send_500(client, "Header generation failed.");
+                        failed = 1;
+                    }
+                } else {
+                    ICECAST_LOG_ERROR("Client buffer reallocation failed. Dropping client.");
+                    client_send_500(client, "Buffer reallocation failed.");
+                    failed = 1;
+                }
+            }
 
-        client->respcode = 200;
-        client_set_queue (client, NULL);
-        client->refbuf = refbuf;
-        refbuf->len = strlen (refbuf->data);
-        fserve_add_client (client, NULL);
+            if (!failed) {
+                  snprintf(refbuf->data + ret, full_len - ret, "Content-Length: %d\r\n\r\n%s", len, string);
+
+                client->respcode = 200;
+                client_set_queue (client, NULL);
+                client->refbuf = refbuf;
+                refbuf->len = strlen (refbuf->data);
+                fserve_add_client (client, NULL);
+            }
+        }
         xmlFree (string);
     }
     else
     {
-        WARN1 ("problem applying stylesheet \"%s\"", xslfilename);
+        ICECAST_LOG_WARN("problem applying stylesheet \"%s\"", xslfilename);
         client_send_404 (client, "XSLT problem");
     }
     thread_mutex_unlock (&xsltlock);
