@@ -1,3 +1,4 @@
+#define _WIN32_WINNT 0x0400
 #include <windows.h>
 #include <stdio.h>
 #include <direct.h>
@@ -24,14 +25,8 @@ SERVICE_STATUS_HANDLE   hStatus;
  
 void  ServiceMain(int argc, char** argv); 
 void  ControlHandler(DWORD request); 
-int InitService();
 extern "C" int mainService(int argc, char **argv);
 
-int InitService() 
-{ 
-   int result = 0;
-   return(result); 
-}
 
 void installService (const char *path)
 {
@@ -44,21 +39,7 @@ void installService (const char *path)
         SC_HANDLE manager = OpenSCManager( NULL, NULL, SC_MANAGER_ALL_ACCESS );
         if (manager == NULL)
 		{
-			LPVOID lpMsgBuf;
-			FormatMessage( 
-					FORMAT_MESSAGE_ALLOCATE_BUFFER | 
-					FORMAT_MESSAGE_FROM_SYSTEM | 
-					FORMAT_MESSAGE_IGNORE_INSERTS,
-					NULL,
-					GetLastError(),
-					MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
-					(LPTSTR) &lpMsgBuf,
-					0,
-					NULL 
-					);
-
-			printf ("OpenSCManager: %s\n", (LPCTSTR)lpMsgBuf);
-			LocalFree( lpMsgBuf );
+            MessageBox (NULL, "OpenSCManager failed", NULL, MB_SERVICE_NOTIFICATION);
 			return;
 		}
 
@@ -79,21 +60,7 @@ void installService (const char *path)
 		);
 		if (service == NULL)
 		{
-			LPVOID lpMsgBuf;
-			FormatMessage( 
-					FORMAT_MESSAGE_ALLOCATE_BUFFER | 
-					FORMAT_MESSAGE_FROM_SYSTEM | 
-					FORMAT_MESSAGE_IGNORE_INSERTS,
-					NULL,
-					GetLastError(),
-					MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
-					(LPTSTR) &lpMsgBuf,
-					0,
-					NULL 
-					);
-
-			printf ("CreateService: %s\n", (LPCTSTR)lpMsgBuf);
-			LocalFree( lpMsgBuf );
+            MessageBox (NULL, "CreateService failed", NULL, MB_SERVICE_NOTIFICATION);
 			CloseServiceHandle (manager);
 			return;
 		}
@@ -108,67 +75,44 @@ void removeService()
 	SC_HANDLE manager = OpenSCManager( NULL, NULL, SC_MANAGER_ALL_ACCESS );
 	if (manager == NULL)
 	{
-		LPVOID lpMsgBuf;
-		FormatMessage( 
-				FORMAT_MESSAGE_ALLOCATE_BUFFER | 
-				FORMAT_MESSAGE_FROM_SYSTEM | 
-				FORMAT_MESSAGE_IGNORE_INSERTS,
-				NULL,
-				GetLastError(),
-				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
-				(LPTSTR) &lpMsgBuf,
-				0,
-				NULL 
-				);
-
-		printf ("OpenSCManager: %s\n", (LPCTSTR)lpMsgBuf);
-		LocalFree( lpMsgBuf );
+        MessageBox (NULL, "OpenSCManager failed", NULL, MB_SERVICE_NOTIFICATION);
 		return;
 	}
 
 	SC_HANDLE service = OpenService (manager, PACKAGE_STRING, DELETE);
 	if (service) {
 		DeleteService(service);
-		printf("Service Removed\n");
         CloseServiceHandle (service);
+        printf ("Service deleted, may require reboot to complete removal\n");
 	}
 	else
 		printf("Service not found\n");
     CloseServiceHandle (manager);
+    Sleep (1500);
 }
 void ControlHandler(DWORD request) 
 { 
    switch(request) { 
       case SERVICE_CONTROL_STOP: 
-		global.running = ICE_HALTING;
-		ServiceStatus.dwWin32ExitCode = 0; 
-		ServiceStatus.dwCurrentState = SERVICE_STOPPED; 
-		SetServiceStatus (hStatus, &ServiceStatus);
-		return; 
+          if (ServiceStatus.dwCurrentState != SERVICE_STOP)
+          {
+              ServiceStatus.dwCurrentState = SERVICE_STOP_PENDING; 
+              SetServiceStatus (hStatus, &ServiceStatus);
+              global.running = ICE_HALTING;
+              return; 
+          }
  
-      case SERVICE_CONTROL_SHUTDOWN: 
-		global.running = ICE_HALTING;
-		ServiceStatus.dwWin32ExitCode = 0; 
-		ServiceStatus.dwCurrentState = SERVICE_STOPPED; 
-		SetServiceStatus (hStatus, &ServiceStatus);
-		return; 
       default:
 		break;
     } 
  
     // Report current status
     SetServiceStatus (hStatus, &ServiceStatus);
- 
-    return; 
 }
 
 void ServiceMain(int argc, char** argv) 
 { 
-   int error; 
- 
-   ServiceStatus.dwServiceType = SERVICE_WIN32; 
-   ServiceStatus.dwCurrentState = SERVICE_START_PENDING; 
-   ServiceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN;
+   ServiceStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
    ServiceStatus.dwWin32ExitCode = 0; 
    ServiceStatus.dwServiceSpecificExitCode = 0; 
    ServiceStatus.dwCheckPoint = 0; 
@@ -177,19 +121,12 @@ void ServiceMain(int argc, char** argv)
    hStatus = RegisterServiceCtrlHandler(PACKAGE_STRING, (LPHANDLER_FUNCTION)ControlHandler); 
    if (hStatus == (SERVICE_STATUS_HANDLE)0) { 
       // Registering Control Handler failed
+      MessageBox (NULL, "RegisterServiceCtrlHandler failed", NULL, MB_SERVICE_NOTIFICATION);
       return; 
    }  
-   // Initialize Service 
-   error = InitService(); 
-   if (error) {
-      // Initialization failed
-      ServiceStatus.dwCurrentState = SERVICE_STOPPED; 
-      ServiceStatus.dwWin32ExitCode = -1; 
-      SetServiceStatus(hStatus, &ServiceStatus); 
-      return; 
-   } 
    // We report the running status to SCM. 
    ServiceStatus.dwCurrentState = SERVICE_RUNNING; 
+   ServiceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP;
    SetServiceStatus (hStatus, &ServiceStatus);
  
    /* Here we do the work */
@@ -205,12 +142,10 @@ void ServiceMain(int argc, char** argv)
         argv2 [2] = argv[1];
     argv2[3] = NULL;
 
-	int ret = mainService(argc2, (char **)argv2);
+	ServiceStatus.dwWin32ExitCode = mainService(argc2, (char **)argv2);
 
 	ServiceStatus.dwCurrentState = SERVICE_STOPPED;
-	ServiceStatus.dwWin32ExitCode = -1;
 	SetServiceStatus(hStatus, &ServiceStatus);
-	return; 
 }
 
 
@@ -224,10 +159,7 @@ int main(int argc, char **argv)
     if (!strcmp(argv[1], "install"))
     {
         if (argc > 2)
-        {
-            printf ("Installing service from %s\n", argv[2]);
             installService(argv[2]);
-        }
         else
             printf ("install requires a path arg as well\n");
         Sleep (2000);
@@ -235,16 +167,15 @@ int main(int argc, char **argv)
     }
     if (!strcmp(argv[1], "remove") || !strcmp(argv[1], "uninstall"))
     {
-        printf ("removing service\n");
         removeService();
-        Sleep (2000);
         return 0;
     }
 
     if (_chdir(argv[1]) < 0)
     {
-        printf ("unable to change to directory %s\n", argv[1]);
-        Sleep (2000);
+        char buffer [256];
+        _snprintf (buffer, sizeof(buffer), "Unable to change to directory %s", argv[1]);
+        MessageBox (NULL, buffer, NULL, MB_SERVICE_NOTIFICATION);
         return 0;
     }
 
@@ -255,7 +186,8 @@ int main(int argc, char **argv)
 	ServiceTable[1].lpServiceName = NULL;
 	ServiceTable[1].lpServiceProc = NULL;
 	// Start the control dispatcher thread for our service
-	StartServiceCtrlDispatcher(ServiceTable);  
+    if (StartServiceCtrlDispatcher(ServiceTable) == 0)
+        MessageBox (NULL, "StartServiceCtrlDispatcher failed", NULL, MB_SERVICE_NOTIFICATION);
 
     return 0;
 }
